mod server;

use clap::Parser;
use server::ServerType;
use std::io;
use std::path::PathBuf;
use std::sync::{Arc, Mutex};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
struct Cli {
    /// Type of server
    type_: ServerType,
    /// Version string for the server, e.g. 1.19.4-545
    server_version: String,
    /// Server jar to execute
    jar: PathBuf,

    /// Directory where configs are stored, and where the server will run [default: .]
    #[arg(long, value_name = "CONFIG_DIR", env = "ALEX_CONFIG_DIR")]
    config: Option<PathBuf>,
    /// Directory where world files will be saved [default: ../worlds]
    #[arg(long, value_name = "WORLD_DIR", env = "ALEX_WORLD_DIR")]
    world: Option<PathBuf>,
    /// Directory where backups will be stored [default: ../backups]
    #[arg(long, value_name = "BACKUP_DIR", env = "ALEX_WORLD_DIR")]
    backup: Option<PathBuf>,
    /// Java command to run the server jar with
    #[arg(long, value_name = "JAVA_CMD", default_value_t = String::from("java"), env = "ALEX_JAVA")]
    java: String,

    /// XMS value in megabytes for the server instance
    #[arg(long, default_value_t = 1024, env = "ALEX_XMS")]
    xms: u64,
    /// XMX value in megabytes for the server instance
    #[arg(long, default_value_t = 2048, env = "ALEX_XMX")]
    xmx: u64,

    /// How many backups to keep
    #[arg(short = 'n', long, default_value_t = 7, env = "ALEX_MAX_BACKUPS")]
    max_backups: u64,
    /// How frequently to perform a backup, in minutes; 0 to disable.
    #[arg(short = 't', long, default_value_t = 0, env = "ALEX_FREQUENCY")]
    frequency: u64,
}

fn backups_thread(counter: Arc<Mutex<server::ServerProcess>>, frequency: u64) {
    loop {
        std::thread::sleep(std::time::Duration::from_secs(frequency * 60));

        {
            let mut server = counter.lock().unwrap();

            // We explicitely ignore the error here, as we don't want the thread to fail
            let _ = server.backup();
        }
    }
}

fn main() {
    let cli = Cli::parse();

    let cmd = server::ServerCommand::new(cli.type_, &cli.server_version)
        .java(&cli.java)
        .jar(cli.jar)
        .config(cli.config.unwrap_or(".".into()))
        .world(cli.world.unwrap_or("../worlds".into()))
        .backup(cli.backup.unwrap_or("../backups".into()))
        .xms(cli.xms)
        .xmx(cli.xmx)
        .max_backups(cli.max_backups);
    let counter = Arc::new(Mutex::new(cmd.spawn().expect("Failed to start server.")));

    if cli.frequency > 0 {
        let clone = Arc::clone(&counter);
        std::thread::spawn(move || backups_thread(clone, cli.frequency));
    }

    let stdin = io::stdin();
    let input = &mut String::new();

    loop {
        input.clear();

        if stdin.read_line(input).is_err() {
            continue;
        };

        {
            let mut server = counter.lock().unwrap();

            if let Err(e) = server.send_command(input) {
                println!("{}", e);
            };
        }

        if input.trim() == "stop" {
            break;
        }
    }
}
